const express = require('express'); //pouziti express frameworku 

const app = express();

const Joi = require('joi'); //pouziti balicku joi urceneho pro validaci vstupnich dat

const mongoose = require('mongoose'); //pouziti balicku mongoose

const PORT = process.env.PORT || 3000;

app.use(express.json()) // funkce parsujici prichozi request

require('dotenv').config()  // ukládání hesel 

//http na https presmerovani
app.enable('trust proxy');
app.use((req,res,next)=>{
    if(req.secure){
        next();
    }else{
        res.redirect('https://bpsklenik.herokuapp.com'+req.url);
        console.log(`Doslo k redirectu na https://bpsklenik.herokuapp.com${req.url}`);
    }
});

const recordSchema = new mongoose.Schema({  //definice schematu pro mongoose
    sklenik: String,
    velicina: String,
    hodnota: Number,
    datum: Date
},
{ 
    versionKey: false // Vypnuti verzovani verze schematu ke kazdemu ulozeni requestu do databaze
}
);

const Record = mongoose.model('Record', recordSchema); // zkompilovani schematu do modelu

function validateRecord(record) // funkce na validaci hodnot
{
    const schema = Joi.object({
        sklenik:        Joi.string().required(),
        velicina:       Joi.string().required(),
        hodnota:        Joi.number().required(),
        datum:          Joi.date().required()
    });
    return schema.validate(record); 
}

function isAuthorized(req, res, next) {  //funkce implementující Basic Auth - overeni z hlavicky enpointu
    const auth = req.headers.authorization; 
    const username = Buffer.from(auth.split(' ')[1], 'base64').toString('ascii').split(':')[0] //rozparsovani, dekodovani uzivatelskeho jmena a ulozeni do promenne
    const password = Buffer.from(auth.split(' ')[1], 'base64').toString('ascii').split(':')[1] // --//-- pro heslo

    if (username === process.env.login_end && password === process.env.password_end) { //overeni uzivatelskeho jmena a hesla
      next();
    } else {
      res.status(401);
      res.send('Access forbidden');
    }
}

mongoose.connect('mongodb+srv://pokkat:'+process.env.pass_mongoDB+'@cluster0.mzpwxam.mongodb.net/?retryWrites=true&w=majority', { useNewUrlParser: true }) // pripojeni k mongoDB
  .then(() => console.log('Connected to MongoDB!'))
  .catch(error => console.error('Could not connect to MongoDB... ', error));
  

app.get('/api/records', isAuthorized, (req, res) => { // GET endpoint ktere vrati vsechny zaznamy z databaze
    Record.find().then(records => { res.json(records) })
});

app.post('/api/records',isAuthorized,  (req, res) => { // POST endpoint, ktery ulozi zaznam do databaze
    const { error } = validateRecord(req.body);
      if (error) {
          res.status(400).send(error.details[0].message);
      } else {
      Record.create(req.body)
        .then(result => { res.json(result) })
        .catch(err => { res.send("Nepodařilo se uložit záznam!") });
      }
  });

app.listen(PORT, () => {
    console.log(`Our app is running on port ${ PORT }`);
});